UNIT RP280;

(* support for FUND2.8 programme *)

INTERFACE

USES
    Crt;

CONST
     NYear = 350;                      (* number of years *)
     NoPer = 18;                       (* number of periods *)
     RepPer = 18;
     NoReg = 16;                       (* number of regions *)
     NOI = 3;                          (* number of instruments *)
     sessionpathname = 'D:\FUND28\DATA\';
     ModeT = 2.5;                      (* modal warming 2xCO2 *)
     BaseT = 3.0;                      (* base case warming *)
     DBsT  = 0.04;                     (* base case yearly warming *)
     DBsTa = 0.025;
     ModeS = 0.30604 (*1.5*);         (* modal sea level rise T=1 *)
     BaseS = 0.50;                     (* base case sea level rise T=2.5*)
     DBsS  = 0.0083;                   (* base case yearly rise *)
     co2pre = 275;
     ch4dec = 1/8.6;                   (* ch4 decay *)
     ch4pre = 790;                     (* ch4 preindustrial *)
     n2odec = 1/120;                   (* n2o decay *)
     n2opre = 285;                     (* n2o preindustrial *)
     lifetemp = 50;                    (* kutzooi *)
     lifesea = 50;
     lifeCO2 = 363;
     lifeCO3 = 74;
     lifeCO4 = 17;
     lifeCO5 = 2;
     nospecbase = 14000000;
     biolossbg = 0.003;
     biolosssd = 0.002;
     biosensbg = 0.001;
     biosenssd = 0.001;
     biosharebg = 0.05;
     biosharesd = 0.05;
     watechratebg = 0.005;
     watechratesd = 0.005;

TYPE
    Year = ARRAY [0..NYear] OF double;
    YrArr = ^Year;
    CtrArr = ARRAY [1..NoPer] OF double;
    RegArr = ARRAY [1..NoReg] OF double;
    RegRec = RECORD
                   decade: word;
                   oda: double;
                   trade: ctrarr;
                   income : double;       (* income *)
                   gdp0 : double;         (* initial income *)
                   ypcgrowth : yrarr;       (* income growth per year *)
                   ecgrsd : double;
                   ecgradd : double;       (* multiplier *)
                   ryg : double;          (* reduction inc growth taxing *)
                   taxmp: double;         (* multiplier *)
                   agrish : double;        (* agricultural share *)
                   agrish0: double;
                   welfare : double;      (* welfare *)
                   basewf : double;       (* Pareto minimal welfare *)
                   population : double;   (* population *)
                   pop0 : double;         (* initial population, needed for reinitialisation *)
                   YpC : double;          (* income per capita *)
                   pgrowth : yrarr;       (* population growth per year *)
                   pgadd : double;        (* uncertainty *)
                   pgsd : double;         (* standard deviation *)
                   urbpop : double;       (* urban population *)
                   urbpop0 : double;
                   urbcorr : double;
                   aeei : yrarr;          (* autonomous energy efficiency *)
                   aeeiadd: double;
                   aeeisd: double;
                   acei : yrarr;          (* autonomous carbon efficiency *)
                   aceiadd: double;
                   aceisd: double;
                   reei : double;           (* reduction energy efficiency *)
                   rcei : double;           (* reduction carbon efficiency *)
                   seei : double;
                   scei : double;
                   htax : double;
                   know : double;
                   emissint : double;       (* GHG emission intensity *)
                   energint : double;       (* energy intensity *)
                   energuse : double;       (* energy use *)
                   emission : double;       (* emissions from fossil fuel *)
                   emiss0   : double;         (* emissions in base year *)
                   cummemm  : double;
                   forestemm: yrarr;       (* emissions from deforestation, MtCO2 *)
                   foremadd : double;
                   foremsd: double;
                   forestry : ctrarr;       (* reduction through forestry *)
                   forconst : double;       (* forestry cost constant *)
                   forlinear : double;      (* forestry cost linear *)
                   forsquare : double;      (* forestry cost square *)
                   forcost : double;        (* forestry costs *)
                   formp : double;          (* multiplier *)
                   ch4em : yrarr;           (* exogenous methane emissions *)
                   n2oem : yrarr;           (* exogenous n2o emissions *)
                   eloss : double;          (* tangible loss *)
                   sloss : double;          (* intangible loss *)
                   kant  : double;

                   dead : double;           (* number of deaths *)
                   deadcost: double;
                   morbcost: double;
                   yll: double;             (* years of life lost *)
                   d2ld: double;
                   d2lm: double;
                   d2ls: double;
                   d2lc: double;
                   d2lr: double;
                   yld: double;             (* years of life diseased *)
                   d2dd: double;
                   d2dm: double;
                   d2ds: double;
                   d2dc: double;
                   d2dr: double;
                   species: double;         (* loss species *)
                   valinc: double;
                   valadj: double;
                   water: double;           (* water *)
                   wrbmbg: double;
                   wrbmsd: double;
                   wrbm: double;
                   watech: double;
                   forests: double;            (* forestry *)
                   forbmbg: double;
                   forbmsd: double;
                   forbm: double;
                   heating: double;             (* heating *)
                   hebmbg: double;
                   hebmsd: double;
                   hebm: double;
                   cumaeei: double;
                   cooling: double;             (* cooling *)
                   cebmbg: double;
                   cebmsd: double;
                   cebm: double;
                   dengue: double;              (* dengue fever *)
                   dfbs: double;
                   dfch: double;
                   dfchbg: double;
                   dfchsd: double;
                   schisto: double;             (* schistosomiasis *)
                   smbs: double;
                   smch: double;
                   smchbg: double;
                   smchsd: double;
                   malaria: double;             (* malaria *)
                   malbs: double;
                   malch: double;
                   malchbg: double;
                   malchsd: double;
                   diadead: double;            (* diarrhea *)
                   diasick: double;
                   diamort: double;
                   diayld: double;
                   agcost: double;             (* agriculture, general *)
                   agrate: double;             (* agriculture, rate *)
                   agrbm: double;
                   agrbmbg: double;
                   agrbmsd: double;
                   agtime: double;
                   agtimebg: double;
                   agtimesd: double;
                   agco2: double;            (* agriculture, fertilisation *)
                   agcbm: double;
                   agcbmbg: double;
                   agcbmsd: double;
                   aglevel: double;             (* agriculture, level *)
                   aglbm: double;
                   aglbmbg: double;
                   aglbmsd: double;
                   agopt: double;
                   agoptbg: double;
                   agoptsd: double;
                   aglparl: double;
                   aglparq: double;
                   plus: double;
                   plus90: double;
                   basecardvasc: double;
                   cardvasc90: double;
                   cardcold: double;            (* cardiovascular, cold *)
                   ccplbmbg: double;
                   ccplbmsd: double;
                   ccplbm: double;
                   ccpqbmbg: double;
                   ccpqbmsd: double;
                   ccpqbm: double;
                   ccmlbmbg: double;
                   ccmlbmsd: double;
                   ccmlbm: double;
                   ccmqbmbg: double;
                   ccmqbmsd: double;
                   ccmqbm: double;

                   cardheat: double;            (* cardiovascular, heat *)
                   chplbmbg: double;
                   chplbmsd: double;
                   chplbm: double;
                   chpqbmbg: double;
                   chpqbmsd: double;
                   chpqbm: double;
                   chmlbmbg: double;
                   chmlbmsd: double;
                   chmlbm: double;
                   chmqbmbg: double;
                   chmqbmsd: double;
                   chmqbm: double;
                   resp: double;            (* respiratory *)
                   rlbmbg: double;
                   rlbmsd: double;
                   rlbm: double;
                   rqbmbg: double;
                   rqbmsd: double;
                   rqbm: double;
                   baseresp: double;
                   resp90: double;
                   wetland: double;             (* wetlands *)
                   wetmeas: double;
                   wettot: double;
                   wetann: double;
                   wetmax: double;
                   wlbmbg: double;
                   wlbmsd: double;
                   wlbm: double;
                   wmbmbg: double;
                   wmbmsd: double;
                   wmbm: double;
                   wvbm: double;
                   wetval: double;
                   npwetcost: double;
                   wetcost: double;
                   wetdel: double;
                   dryland: double;             (* dryland *)
                   dlbmbg: double;
                   dlbmsd: double;
                   dlbm: double;
                   dvbm: double;
                   dryval: double;
                   npdrycost: double;
                   drycost: double;
                   drydel: double;
                   protlev: double;             (* protection level *)
                   protcost: double;            (* protection costs *)
                   npprotcost: double;
                   pcbg: double;
                   pcsd: double;
                   pc: double;
                   coast: double;               (* coast length *)
                   popdens: double;             (* population density *)
                   coastpd: double;             (* pop dens at coast, relative to total *)
                   area: double;
                   leave: double;
                   cumleave: double;
                   enter: double;
                   cumenter: double;
                   immigratebg: ARRAY [1..NoReg] OF double;
                   immigratesd: ARRAY [1..NoReg] OF double;
                   immigrate: ARRAY [1..NoReg] OF double;
                   leavecost: double;
                   entercost: double;

                   pop90: double;
                   GDP90: double;
                   ypc90: double;

(* parameters *)
                   NPVW : double;           (* net present value welfare *)
                   hNPVW: double;           (* help npv welfare *)
                   discrate : double;       (* discount rate *)
                   cdr : double;            (* composite discount rate *)
                   tax : ctrarr;            (* carbon tax *)
                   taxpar: double;          (* tax cost parameter *)
                   ch4red: ctrarr;          (* methane reduction *)
                   ch4par1: double;
                   ch4par2: double;
                   ch4cost: double;
                   n2ored: ctrarr;
                   n2opar1: double;
                   n2opar2: double;
                   n2ocost: double;
                   so2: double;
                   regtmp: double;
                   regdt: double;
                   bregtmp: double;
                   temp90: double;
                   scentemp: yrarr;
                   name : string;           (* name *)
             END;
    EconArr = ARRAY [1..NoReg] OF RegRec;

    Box = ARRAY [1..5] OF double;
    ClimRec = RECORD
                    acco2 : yrarr;           (* carbon dioxide *)
                    cbox  : box;             (* co2 boxes *)
                    decay : box;
                    frac  : box;
                    acch4 : yrarr;           (* atm. conc. methane *)
                    acn2o : yrarr;           (* atm. conc. nitrous oxide *)
                    addco2: yrarr;           (* additional co2 emissions *)
                    addch4: yrarr;
                    addn2o: yrarr;
                    ch4add: double;          (* ch4 uncertainty *)
                    ch4sd: double;
                    ch4decay: double;
                    n2oadd: double;          (* n2o uncertainty *)
                    n2osd: double;
                    n2odecay: double;
                    temps : double;          (* temperature sensitivity *)
                    delaytemp: double;
                    seas  : double;          (* sea level sensitivity *)
                    delaysea: double;
                    temp  : double;          (* temperature *)
                    dt    : double;          (* temp difference *)
                    sea   : double;          (* sea level *)
                    ds    : double;          (* sea difference *)
                    rfco2 : double;          (* radiative forcing CO2 *)
                    rfch4 : double;          (* rad. forc. CH4 *)
                    rfn2o : double;          (* rad. forc. N2O *)
                    rfso2 : double;
                    ch4n2o: double;          (* interaction term pre-indus.
                                                rad. forc. CH4-N2O *)
                    globso2: double;
                    radforc: double;
                    biodiv: double
              END;

VAR
   Economy : EconArr;
   Climate : ClimRec;
   rho20, rhocorr: double;
   ncemmres: ARRAY [1..NoReg, 1..NoPer] OF real;
   cemmres: ARRAY [1..NoPer] OF real;


PROCEDURE ReadWater;
(* reads benchmark water resources costs from water.001 *)

PROCEDURE ReadForestry;
(* reads benchmark forestry costs from forestry.001 *)

PROCEDURE ReadHeating;
(* reads benchmark heating costs from heating.001 *)

PROCEDURE ReadCooling;
(* reads benchmark cooling costs from cooling.001 *)

PROCEDURE ReadDengue;
(* reads benchmark dengue fever from dengue.001 *)

PROCEDURE ReadSchisto;
(* reads benchmark schistosomiasis from schisto.001 *)

PROCEDURE ReadMalaria;
(* reads benchmark malaria from malaria.001 *)

PROCEDURE ReadDiarrhea;
(* reads benchmark dengue fever from diarrhea.001 *)

PROCEDURE ReadCardHeat;
(* reads life-losses from cardheat.001 *)

PROCEDURE ReadCardCold;
(* reads life-losses from cardcold.001 *)

PROCEDURE ReadResp;
(* reads life-losses from resp.001 *)

PROCEDURE ReadYLL;
(* reads dead-to-years-of-life-lost transformation parameters *)

PROCEDURE ReadYLD;
(* reads dead-to-years-of-life-disabled transformation parameters *)

PROCEDURE ReadWetland;
(* reads wetland losses sea level rise from wetland.001 *)

PROCEDURE ReadDryland;
(* reads dryland losses sea level rise from dryland.001 *)

PROCEDURE ReadProtect;
(* reads protection costs sea level rise from protect.001 *)

PROCEDURE ReadImmigrate;
(* reads migration patterns from migrate.001 *)

PROCEDURE ReadYpCGrowth;
(* reads economic growth from ypcgrowth.001 *)

PROCEDURE ReadAgrish;
(* reads share of agriculture from agrish.001 *)

PROCEDURE ReadPGrowth;
(* reads population growth from pgrowth.001 *)

PROCEDURE ReadAEEI;
(* reads autonomous energy efficiency improvement from aeei.001 *)

PROCEDURE ReadACEI;
(* reads autonomous carbon efficiency improvements from acei.001 *)

PROCEDURE ReadForestemm;
(* reads emission from deforestation from forestem.001 *)

PROCEDURE ReadForParam;
(* reads reduction cost parameter for forestry from forparam.001 *)

PROCEDURE ReadDiscRate;
(* reads discount rates from discrate.001 *)

PROCEDURE ReadAgricult;
(* reads agricultural losses from agricult.001 *)

PROCEDURE ReadIncome;
(* reads base income from inc0.001 *)

PROCEDURE ReadPopulation;
(* reads initial population from pop0.001 *)

PROCEDURE ReadAddCO2;
(* reads additional emissions CO2 from addco2.001 *)

PROCEDURE ReadAddCH4;
(* reads additional emissions CH4 from addch4.001 *)

PROCEDURE ReadAddN2O;
(* reads additional emissions N2O from addn2o.001 *)

PROCEDURE ReadCH4Em;
(* reads CH4 emissions from ch4em.001 *)

PROCEDURE ReadN2OEm;
(* reads N2O emissions from n2oem.001 *)

PROCEDURE ReadScenGMT;
(* reads exogenous temperatures from scengmt.001 *)

PROCEDURE ReadBaseRegTemp;
(* reads base regional temperature parameters from bregtmp.001 *)

PROCEDURE ReadNCEmmRestr;
(* reads emission restrictions from ncemmres.001 *)

PROCEDURE ReadCEmmRestr;
(* reads emission restrictions from cemmres.001 *)

PROCEDURE ReadScenUnc;
(* reads uncertainty about scenarios from scenunc.001 *)

PROCEDURE ReadMethPar;
(* reads methane parameters from methpar.001 *)

PROCEDURE ReadNitrOxPar;
(* reads nitrous oxide parameters from nitroxpar.001 *)

FUNCTION Log(x: double): double;
(* returns ln(x) *)

FUNCTION Expo(x: double): double;
(* returns exp(x) *)

FUNCTION Root(x: double): double;
(* returns sqrt(x) *)

FUNCTION Quad(x: double): double;
(* returns x^2 *)

FUNCTION Pow(x,y: double): double;
(* returns x^y *)

IMPLEMENTATION

PROCEDURE ReadDiscRate;
(* reads discount rates from discrate.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'discrate.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].discrate);
     Close(fil)
END;

PROCEDURE ReadWater;
(* reads benchmark water resources costs from water.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'water.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,wrbmbg,wrbmsd);
     Close(fil)
END;

PROCEDURE ReadHeating;
(* reads benchmark heating costs from heating.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'heating.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,hebmbg,hebmsd);
     Close(fil)
END;

PROCEDURE ReadCooling;
(* reads benchmark cooling costs from cooling.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'cooling.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,cebmbg,cebmsd);
     Close(fil)
END;

PROCEDURE ReadForestry;
(* reads benchmark forestry costs from forestry.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'forestry.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,forbmbg,forbmsd);
     Close(fil)
END;

PROCEDURE ReadDengue;
(* reads benchmark dengue fever from dengue.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'dengue.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,dfbs,dfchbg,dfchsd);
     Close(fil)
END;

PROCEDURE ReadSchisto;
(* reads benchmark schistosomiasis from schisto.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'schisto.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,smbs,smchbg,smchsd);
     Close(fil)
END;

PROCEDURE ReadMalaria;
(* reads benchmark malaria from malaria.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'malaria.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,malbs,malchbg,malchsd);
     Close(fil)
END;

PROCEDURE ReadDiarrhea;
(* reads benchmark dengue fever from diarrhea.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'diarrhea.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              ReadLn(fil,diamort,diayld);
              diamort:= diamort*1000;
              diayld:= diayld*1000
         END;
     Close(fil)
END;

PROCEDURE ReadCardHeat;
(* reads life-losses from cardheat.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'cardheat.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              Readln(fil,chmlbmbg,chmlbmsd,chmqbmbg,chmqbmsd,chplbmbg,chplbmsd,chpqbmbg,chpqbmsd);
     Close(fil)
END;

PROCEDURE ReadCardCold;
(* reads life-losses from cardcold.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'cardcold.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              Readln(fil,ccmlbmbg,ccmlbmsd,ccmqbmbg,ccmqbmsd,ccplbmbg,ccplbmsd,ccpqbmbg,ccpqbmsd);
     Close(fil)
END;

PROCEDURE ReadResp;
(* reads life-losses from resp.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'resp.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              Readln(fil,rlbmbg,rlbmsd,rqbmbg,rqbmsd);
     Close(fil)
END;

PROCEDURE ReadYLL;
(* reads dead-to-years-of-life-lost transformation parameters *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'yll.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,d2lm,d2ls,d2ld,d2lc,d2lr);
     Close(fil)
END;

PROCEDURE ReadYLD;
(* reads dead-to-years-of-life-lost transformation parameters *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'yld.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,d2dm,d2ds,d2dd,d2dc,d2dr);
     Close(fil)
END;

PROCEDURE ReadWetland;
(* reads wetland losses sea level rise from wetland.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'wetland.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,wlbmbg,wlbmsd,wmbmbg,wmbmsd,wetmax);
     Close(fil)
END;

PROCEDURE ReadDryland;
(* reads dryland losses sea level rise from dryland.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'dryland.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,dlbmbg,dlbmsd);
     Close(fil)
END;

PROCEDURE ReadProtect;
(* reads protection costs sea level rise from protect.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'protect.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,coast,pcbg,pcsd);
     Close(fil)
END;

PROCEDURE ReadImmigrate;
(* reads migration patterns from migrate.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'migrate.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO BEGIN
         WITH economy[i] DO
              FOR j:= 1 TO NoReg DO
                  Read(fil,immigratebg[j]);
         ReadLn(fil)
     END;
     ReadLn(fil);
     FOR i:= 1 TO NoReg DO BEGIN
         WITH economy[i] DO
              FOR j:= 1 TO NoReg DO
                  Read(fil,immigratesd[j]);
         ReadLn(fil)
     END;
     close(fil)
END;

PROCEDURE ReadYpCGrowth;
(* reads economic growth from ypcgrowth.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'ypcgrowth.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].ypcgrowth^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadAgrish;
(* reads share of agriculture from agrish.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'agrish.001');
     Reset(fil);
     FOR j:= 1 TO NoReg DO
         ReadLn(fil,Economy[j].agrish0);
     Close(fil)
END;

PROCEDURE ReadPGrowth;
(* reads population growth from pgrowth.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'pgrowth.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].pgrowth^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadAEEI;
(* reads autonomous energy efficiency improvement from aeei.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'aeei.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].aeei^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadACEI;
(* reads autonomous carbon efficiency improvements from acei.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'acei.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].acei^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadForestemm;
(* reads emission from deforestation from forestemm.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'forestemm.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].forestemm^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadForParam;
(* reads reduction cost parameter for forestry from forparam.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'forparam.001');
     Reset(fil);
     FOR j:= 1 TO NoReg DO BEGIN
         WITH Economy[j] DO
              Read(fil,forconst,forlinear,forsquare);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadAgricult;
(* reads agricultural losses global warming from agricult.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'agricult.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,agrbmbg,agrbmsd,aglbmbg,aglbmsd,agoptbg,agoptsd,agtimebg,agtimesd,agcbmbg,agcbmsd);
     Close(fil)
END;

PROCEDURE ReadIncome;
(* reads base income from inc0.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'inc0.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].GDP0);
     Close(fil)
END;

PROCEDURE ReadPopulation;
(* reads initial population from pop0.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'pop0.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].pop0);
     Close(fil)
END;

PROCEDURE ReadAddCO2;
(* reads additional emissions CO2 from addco2.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'addco2.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO
         ReadLn(fil,climate.addco2^[i]);
     Close(fil)
END;

PROCEDURE ReadAddCH4;
(* reads additional emissions CH4 from addch4.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'addch4.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO
         ReadLn(fil,climate.addch4^[i]);
     Close(fil)
END;

PROCEDURE ReadAddN2O;
(* reads additional emissions N2O from addn2o.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'addn2o.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO
         ReadLn(fil,climate.addn2o^[i]);
     Close(fil);
     FOR i:= 0 TO NYear DO
         climate.addn2o^[i]:= climate.addn2o^[i]*14/22

END;

PROCEDURE ReadCH4Em;
(* reads exogenous emissions CH4 from ch4em.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'ch4em.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,economy[j].ch4em^[i]);
         Readln(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadN2OEm;
(* reads exogenous N2O emissions from n2oem.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'n2oem.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,economy[j].n2oem^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadScenGMT;
(* reads exogenous temperatures from scengmt.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'scengmt.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,economy[j].scentemp^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadNCEmmRestr;
(* reads emission restrictions from ncemmres.001 *)

VAR fil: text;
    i,j: word;
    hulp: double;

BEGIN
     assign(fil,sessionpathname+'ncemmres.001');
     reset(fil);
     FOR i:= 1 TO NoPer DO BEGIN
         FOR j:= 1 TO NoReg DO BEGIN
             read(fil,hulp);
             ncemmres[j,i]:= hulp*1000
         END;
         readln(fil)
     END;
     close(fil)
END;

PROCEDURE ReadCEmmRestr;
(* reads emission restrictions from cemmres.001 *)

VAR fil: text;
    i,j: word;
    hulp: double;

BEGIN
     assign(fil,sessionpathname+'cemmres.001');
     reset(fil);
     FOR i:= 1 TO NoPer DO BEGIN
         readln(fil,hulp);
         cemmres[i]:= hulp*1000
     END;
     close(fil)
END;

PROCEDURE ReadScenUnc;
(* reads uncertainty about scenarios from scenunc.001 *)

VAR fil: text;
    i,j: word;
    hulp: double;

BEGIN
     assign(fil,sessionpathname+'scenunc.001');
     reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
             readln(fil,pgsd,ecgrsd,aeeisd,aceisd,foremsd);
             pgsd:= pgsd/100; (* perhaps times 1.5 *)
             ecgrsd:= ecgrsd/100;
             aeeisd:= aeeisd/100;
             aceisd:= aceisd/100;
             foremsd:= foremsd/100
         END;
     close(fil)
END;

PROCEDURE ReadMethPar;
(* reads methane parameters from methpar.001 *)

VAR fil: text;
    i,j: word;
    hulp: double;

BEGIN
     assign(fil,sessionpathname+'methpar.001');
     reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              readln(fil,ch4par1,hulp,ch4par2,hulp);
              ch4par1:= exp(ch4par1)
         END;
     close(fil)
END;

PROCEDURE ReadNitrOxPar;
(* reads nitrous oxide parameters from nitroxpar.001 *)

VAR fil: text;
    i,j: word;
    hulp: double;

BEGIN
     assign(fil,sessionpathname+'nitrox.001');
     reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              readln(fil,n2opar1,hulp,n2opar2,hulp);
              n2opar1:= exp(n2opar1)
         END;
     close(fil)
END;

PROCEDURE ReadBaseRegTemp;
(* reads base regional temperature parameters from bregtmp.001 *)

VAR fil: text;
    i,j: word;
    hulp: double;

BEGIN
     assign(fil,sessionpathname+'bregtmp.001');
     reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              readln(fil,bregtmp,temp90);
     close(fil)
END;


FUNCTION Log(x: double): double;
(* returns ln(x) *)

BEGIN
     IF (x > 1.0E-308) AND (x < 1.0E308) THEN
        log:= ln(x)
     ELSE IF x < 1.0E-308 THEN
          log:= ln(1.0E-308)
     ELSE
         log:= ln(1.0E38)
END;

FUNCTION Expo(x: double): double;
(* returns exp(x) *)

BEGIN
     IF (x < 709) AND (x > -11356) THEN
        expo:= exp(x)
     ELSE IF x < -11356 THEN
          expo:= exp(-11356)
     ELSE
         expo:= exp(709)
END;

FUNCTION Root(x: double): double;
(* returns sqrt(x) *)

BEGIN
     IF (x < 1.0E308) AND (x > 0.00000000001) THEN
        root:= sqrt(x)
     ELSE IF x < 0.00000000001 THEN
          root:= 0
     ELSE
         root:= 1.0E154
END;

FUNCTION Quad(x: double): double;
(* returns x^2 *)

BEGIN
     IF x < 1.0E154 THEN
        quad:= sqr(x)
     ELSE
         quad:= 1.0E308
END;

FUNCTION Pow(x,y: double): double;
(* returns x^y *)

BEGIN
     pow:= expo(y*log(x))
END;

END.